@@ -28,11 +28,6 @@
 #include <raptor_config.h>
 #endif
 
-#ifdef WIN32
-#include <win32_raptor_config.h>
-#endif
-
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -49,7 +44,7 @@
 #include <math.h>
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 #ifndef STANDALONE
@@ -81,11 +76,6 @@ struct raptor_turtle_writer_s {
 
   /* indentation per level if formatting */
   int indent;
-
-  raptor_uri* xsd_boolean_uri;
-  raptor_uri* xsd_decimal_uri;
-  raptor_uri* xsd_double_uri;
-  raptor_uri* xsd_integer_uri;
 };
 
 
@@ -127,7 +117,8 @@ raptor_turtle_writer_newline(raptor_turtle_writer *turtle_writer)
 
   while(num_spaces > 0) {
     int count;
-    count = (num_spaces > (int)SPACES_BUFFER_SIZE) ? (int)SPACES_BUFFER_SIZE : num_spaces;
+    count = (num_spaces > RAPTOR_GOOD_CAST(int, SPACES_BUFFER_SIZE)) ? 
+            RAPTOR_GOOD_CAST(int, SPACES_BUFFER_SIZE) : num_spaces;
 
     raptor_iostream_counted_string_write(spaces_buffer, count, turtle_writer->iostr);
 
@@ -165,8 +156,8 @@ raptor_new_turtle_writer(raptor_world* world,
   
   raptor_world_open(world);
 
-  turtle_writer = (raptor_turtle_writer*)RAPTOR_CALLOC(raptor_turtle_writer, 1,
-                                                       sizeof(*turtle_writer));
+  turtle_writer = RAPTOR_CALLOC(raptor_turtle_writer*, 1,
+                                sizeof(*turtle_writer));
 
   if(!turtle_writer)
     return NULL;
@@ -177,7 +168,7 @@ raptor_new_turtle_writer(raptor_world* world,
 
   turtle_writer->nstack = nstack;
   if(!turtle_writer->nstack) {
-    turtle_writer->nstack = nstack = raptor_new_namespaces(world, 1);
+    turtle_writer->nstack = raptor_new_namespaces(world, 1);
     turtle_writer->my_nstack = 1;
   }
 
@@ -192,11 +183,6 @@ raptor_new_turtle_writer(raptor_world* world,
     raptor_turtle_writer_base(turtle_writer, base_uri);
   turtle_writer->base_uri = base_uri;
 
-  turtle_writer->xsd_boolean_uri = raptor_new_uri(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#boolean");
-  turtle_writer->xsd_decimal_uri = raptor_new_uri(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
-  turtle_writer->xsd_double_uri = raptor_new_uri(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#double");
-  turtle_writer->xsd_integer_uri = raptor_new_uri(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
-  
   return turtle_writer;
 }
 
@@ -217,15 +203,6 @@ raptor_free_turtle_writer(raptor_turtle_writer* turtle_writer)
   if(turtle_writer->nstack && turtle_writer->my_nstack)
     raptor_free_namespaces(turtle_writer->nstack);
 
-  if(turtle_writer->xsd_boolean_uri)
-    raptor_free_uri(turtle_writer->xsd_boolean_uri);
-  if(turtle_writer->xsd_decimal_uri)
-    raptor_free_uri(turtle_writer->xsd_decimal_uri);
-  if(turtle_writer->xsd_double_uri)
-    raptor_free_uri(turtle_writer->xsd_double_uri);
-  if(turtle_writer->xsd_integer_uri)
-    raptor_free_uri(turtle_writer->xsd_integer_uri);
-
   RAPTOR_FREE(raptor_turtle_writer, turtle_writer);
 }
 
@@ -323,24 +300,17 @@ raptor_turtle_writer_base(raptor_turtle_writer* turtle_writer,
  * @turtle_writer: Turtle writer object
  * @uri: URI to write
  *
- * Write a URI to the Turtle writer.
+ * Write a Turtle-encoded URI to the Turtle writer.
  *
+ * Return value: non-0 on failure
  **/
-void
+int
 raptor_turtle_writer_reference(raptor_turtle_writer* turtle_writer, 
                                raptor_uri* uri)
 {
-  unsigned char* uri_str;
-  size_t length;
-  
-  uri_str = raptor_uri_to_relative_counted_uri_string(turtle_writer->base_uri, uri, &length);
-
-  raptor_iostream_write_byte('<', turtle_writer->iostr);
-  if(uri_str)
-    raptor_string_ntriples_write(uri_str, length, '>', turtle_writer->iostr);
-  raptor_iostream_write_byte('>', turtle_writer->iostr);
-
-  RAPTOR_FREE(cstring, uri_str);
+  return raptor_uri_escaped_write(uri, turtle_writer->base_uri, 
+                                  RAPTOR_ESCAPED_WRITE_TURTLE_URI,
+                                  turtle_writer->iostr);
 }
 
 
@@ -371,112 +341,6 @@ raptor_turtle_writer_qname(raptor_turtle_writer* turtle_writer,
 }
 
 
-/**
- * raptor_string_python_write:
- * @string: UTF-8 string to write
- * @len: length of UTF-8 string
- * @delim: Terminating delimiter character for string (such as " or >)
- * or \0 for no escaping.
- * @flags: flags 0=N-Triples mode, 1=Turtle (allow raw UTF-8), 2=Turtle long string (allow raw UTF-8), 3=JSON
- * @iostr: #raptor_iostream to write to
- *
- * Write a UTF-8 string using Python-style escapes (N-Triples, Turtle, JSON) to an iostream.
- * 
- * Return value: non-0 on failure such as bad UTF-8 encoding.
- **/
-int
-raptor_string_python_write(const unsigned char *string,
-                           size_t len,
-                           const char delim,
-                           int flags,
-                           raptor_iostream *iostr)
-{
-  unsigned char c;
-  int unichar_len;
-  raptor_unichar unichar;
-
-  if(flags < 0 || flags > 3)
-    return 1;
-  
-  for(; (c=*string); string++, len--) {
-    if((delim && c == delim && (delim == '\'' || delim == '"')) ||
-       c == '\\') {
-      raptor_iostream_write_byte('\\', iostr);
-      raptor_iostream_write_byte(c, iostr);
-      continue;
-    }
-    if(delim && c == delim) {
-      raptor_iostream_counted_string_write("\\u", 2, iostr);
-      raptor_iostream_hexadecimal_write(c, 4, iostr);
-      continue;
-    }
-    
-    if(flags != 2) {
-      /* N-Triples, Turtle or JSON */
-
-      /* Note: NTriples is ASCII */
-      if(c == 0x09) {
-        raptor_iostream_counted_string_write("\\t", 2, iostr);
-        continue;
-      } else if((flags == 3) && c == 0x08) {
-        /* JSON has \b for backspace */
-        raptor_iostream_counted_string_write("\\b", 2, iostr);
-        continue;
-      } else if(c == 0x0a) {
-        raptor_iostream_counted_string_write("\\n", 2, iostr);
-        continue;
-      } else if((flags == 3) && c == 0x0b) {
-        /* JSON has \f for formfeed */
-        raptor_iostream_counted_string_write("\\f", 2, iostr);
-        continue;
-      } else if(c == 0x0d) {
-        raptor_iostream_counted_string_write("\\r", 2, iostr);
-        continue;
-      } else if(c < 0x20|| c == 0x7f) {
-        raptor_iostream_counted_string_write("\\u", 2, iostr);
-        raptor_iostream_hexadecimal_write(c, 4, iostr);
-        continue;
-      } else if(c < 0x80) {
-        raptor_iostream_write_byte(c, iostr);
-        continue;
-      }
-    } else if(c < 0x80) {
-      /* Turtle long string has no escapes except delim */
-      raptor_iostream_write_byte(c, iostr);
-      continue;
-    } 
-    
-    /* It is unicode */
-    
-    unichar_len = raptor_unicode_utf8_string_get_char(string, len, NULL);
-    if(unichar_len < 0 || unichar_len > (int)len)
-      /* UTF-8 encoding had an error or ended in the middle of a string */
-      return 1;
-
-    if(flags >= 1 && flags <= 3) {
-      /* Turtle and JSON are UTF-8 - no need to escape */
-      raptor_iostream_counted_string_write(string, unichar_len, iostr);
-    } else {
-      unichar_len = raptor_unicode_utf8_string_get_char(string, len, &unichar);
-
-      if(unichar < 0x10000) {
-        raptor_iostream_counted_string_write("\\u", 2, iostr);
-        raptor_iostream_hexadecimal_write(unichar, 4, iostr);
-      } else {
-        raptor_iostream_counted_string_write("\\U", 2, iostr);
-        raptor_iostream_hexadecimal_write(unichar, 8, iostr);
-      }
-    }
-    
-    unichar_len--; /* since loop does len-- */
-    string += unichar_len; len -= unichar_len;
-
-  }
-
-  return 0;
-}
-
-
 /**
  * raptor_turtle_writer_quoted_counted_string:
  * @turtle_writer: Turtle writer object
@@ -492,27 +356,32 @@ raptor_turtle_writer_quoted_counted_string(raptor_turtle_writer* turtle_writer,
                                            const unsigned char *s, size_t len)
 {
   const unsigned char *quotes = (const unsigned char *)"\"\"\"\"";
-  const unsigned char *q;
-  size_t q_len;
-  int flags;
-  
+  const unsigned char *q = quotes + 2;
+  size_t q_len = 1;
+  int flags = RAPTOR_ESCAPED_WRITE_TURTLE_LITERAL;
+  int rc = 0;
+
   if(!s)
     return 1;
   
   /* Turtle """longstring""" (2) or "string" (1) */
-  flags = raptor_turtle_writer_contains_newline(s) ? 2 : 1;
-  q = (flags == 2) ? quotes : quotes+2;
-  q_len = (q == quotes) ? 3 : 1;
+  if(raptor_turtle_writer_contains_newline(s)) {
+    /* long string */
+    flags = RAPTOR_ESCAPED_WRITE_TURTLE_LONG_LITERAL;
+    q = quotes;
+    q_len = 3;
+  }
+
   raptor_iostream_counted_string_write(q, q_len, turtle_writer->iostr);
-  raptor_string_python_write(s, strlen((const char*)s), '"', flags,
-                             turtle_writer->iostr);
+  rc = raptor_string_escaped_write(s, strlen((const char*)s), '"', 
+                                   flags, turtle_writer->iostr);
   raptor_iostream_counted_string_write(q, q_len, turtle_writer->iostr);
 
-  return 0;
+  return rc;
 }
 
 
-/**
+/*
  * raptor_turtle_writer_literal:
  * @turtle_writer: Turtle writer object
  * @nstack: Namespace stack for making a QName for datatype URI
@@ -520,7 +389,7 @@ raptor_turtle_writer_quoted_counted_string(raptor_turtle_writer* turtle_writer,
  * @lang: language tag (may be NULL)
  * @datatype: datatype URI (may be NULL)
  *
- * Write a literal (possibly with lang and datatype) to the Turtle writer.
+ * INTERNAL - Write a literal (possibly with lang and datatype) to the Turtle writer.
  *
  * Return value: non-0 on failure
  **/
@@ -542,7 +411,7 @@ raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer,
   /* typed literal special cases */
   if(datatype) {
     /* integer */
-    if(raptor_uri_equals(datatype, turtle_writer->xsd_integer_uri)) {
+    if(raptor_uri_equals(datatype, turtle_writer->world->xsd_integer_uri)) {
       /* FIXME. Work around that gcc < 4.5 cannot disable warn_unused_result */
       long gcc_is_stupid = strtol((const char*)s, &endptr, 10);
       if(endptr != (char*)s && !*endptr) {
@@ -555,8 +424,8 @@ raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer,
       }
 
     /* double, decimal */
-    } else if(raptor_uri_equals(datatype, turtle_writer->xsd_double_uri) ||
-      raptor_uri_equals(datatype, turtle_writer->xsd_decimal_uri)) {
+    } else if(raptor_uri_equals(datatype, turtle_writer->world->xsd_double_uri) ||
+      raptor_uri_equals(datatype, turtle_writer->world->xsd_decimal_uri)) {
       /* FIXME. Work around that gcc < 4.5 cannot disable warn_unused_result */
       double gcc_is_doubly_stupid = strtod((const char*)s, &endptr);
       if(endptr != (char*)s && !*endptr) {
@@ -569,7 +438,7 @@ raptor_turtle_writer_literal(raptor_turtle_writer* turtle_writer,
       }
 
     /* boolean */
-    } else if(raptor_uri_equals(datatype, turtle_writer->xsd_boolean_uri)) {
+    } else if(raptor_uri_equals(datatype, turtle_writer->world->xsd_boolean_uri)) {
       if(!strcmp((const char*)s, "0") || !strcmp((const char*)s, "false")) {
         raptor_iostream_string_write("false", turtle_writer->iostr);
         written = 1;
@@ -700,6 +569,8 @@ raptor_turtle_writer_set_option(raptor_turtle_writer *turtle_writer,
       
     /* Shared */
     case RAPTOR_OPTION_NO_NET:
+    case RAPTOR_OPTION_NO_FILE:
+    case RAPTOR_OPTION_LOAD_EXTERNAL_ENTITIES:
 
     /* XML writer options */
     case RAPTOR_OPTION_RELATIVE_URIS:
@@ -725,10 +596,14 @@ raptor_turtle_writer_set_option(raptor_turtle_writer *turtle_writer,
     /* WWW option */
     case RAPTOR_OPTION_WWW_HTTP_CACHE_CONTROL:
     case RAPTOR_OPTION_WWW_HTTP_USER_AGENT:
+    case RAPTOR_OPTION_WWW_CERT_FILENAME:
+    case RAPTOR_OPTION_WWW_CERT_TYPE:
+    case RAPTOR_OPTION_WWW_CERT_PASSPHRASE:
+    case RAPTOR_OPTION_WWW_SSL_VERIFY_PEER:
+    case RAPTOR_OPTION_WWW_SSL_VERIFY_HOST:
       
     default:
       return -1;
-      break;
   }
 
   return 0;
@@ -818,6 +693,8 @@ raptor_turtle_writer_get_option(raptor_turtle_writer *turtle_writer,
 
     /* Shared */
     case RAPTOR_OPTION_NO_NET:
+    case RAPTOR_OPTION_NO_FILE:
+    case RAPTOR_OPTION_LOAD_EXTERNAL_ENTITIES:
 
     /* XML writer options */
     case RAPTOR_OPTION_RELATIVE_URIS:
@@ -843,6 +720,11 @@ raptor_turtle_writer_get_option(raptor_turtle_writer *turtle_writer,
     /* WWW option */
     case RAPTOR_OPTION_WWW_HTTP_CACHE_CONTROL:
     case RAPTOR_OPTION_WWW_HTTP_USER_AGENT:
+    case RAPTOR_OPTION_WWW_CERT_FILENAME:
+    case RAPTOR_OPTION_WWW_CERT_TYPE:
+    case RAPTOR_OPTION_WWW_CERT_PASSPHRASE:
+    case RAPTOR_OPTION_WWW_SSL_VERIFY_PEER:
+    case RAPTOR_OPTION_WWW_SSL_VERIFY_HOST:
       
     default:
       break;
@@ -870,6 +752,103 @@ raptor_turtle_writer_get_option_string(raptor_turtle_writer *turtle_writer,
   return NULL;
 }
 
+
+/**
+ * raptor_turtle_writer_bnodeid:
+ * @turtle_writer: Turtle writer object
+ * @bnodeid: blank node ID to write
+ * @len: length of @bnodeid
+ *
+ * Write a blank node ID with leading _: to the Turtle writer.
+ *
+ **/
+void
+raptor_turtle_writer_bnodeid(raptor_turtle_writer* turtle_writer,
+                             const unsigned char *bnodeid, size_t len)
+{
+  raptor_bnodeid_ntriples_write(bnodeid, len,
+                                turtle_writer->iostr);
+}
+
+
+/**
+ * raptor_turtle_writer_uri:
+ * @turtle_writer: Turtle writer object
+ * @uri: uri
+ *
+ * Write a #raptor_uri to a turtle writer in qname or URI form
+ *
+ * Return value: non-0 on failure
+ */
+int
+raptor_turtle_writer_uri(raptor_turtle_writer* turtle_writer,
+                         raptor_uri* uri)
+{
+  raptor_qname* qname;
+  int rc = 0;
+
+  if(!uri)
+    return 1;
+
+  qname = raptor_new_qname_from_namespace_uri(turtle_writer->nstack, uri, 10);
+
+  /* XML Names allow leading '_' and '.' anywhere but Turtle does not */
+  if(qname && !raptor_turtle_is_legal_turtle_qname(qname)) {
+    raptor_free_qname(qname);
+    qname = NULL;
+  }
+
+  if(qname) {
+    raptor_turtle_writer_qname(turtle_writer, qname);
+    raptor_free_qname(qname);
+  } else {
+    rc = raptor_turtle_writer_reference(turtle_writer, uri);
+  }
+
+  return rc;
+}
+
+
+/**
+ * raptor_turtle_writer_term:
+ * @turtle_writer: Turtle writer object
+ * @term: term
+ *
+ * Write a #raptor_term to a turtle write
+ *
+ * Return value: non-0 on failure
+ */
+int
+raptor_turtle_writer_term(raptor_turtle_writer* turtle_writer,
+                          raptor_term* term)
+{
+  int rc = 0;
+
+  if(!term)
+    return 1;
+
+  if(term->type == RAPTOR_TERM_TYPE_URI) {
+    rc = raptor_turtle_writer_uri(turtle_writer, term->value.uri);
+  } else if(term->type == RAPTOR_TERM_TYPE_LITERAL) {
+    rc = raptor_turtle_writer_literal(turtle_writer,
+                                      turtle_writer->nstack,
+                                      term->value.literal.string,
+                                      term->value.literal.language, 
+                                      term->value.literal.datatype);
+  } else if(term->type == RAPTOR_TERM_TYPE_BLANK) {
+    rc = raptor_bnodeid_ntriples_write(term->value.blank.string,
+                                       term->value.blank.string_len,
+                                       turtle_writer->iostr);
+  } else {
+    rc = 2;
+  }
+  
+  return rc;
+}
+
+
+
+
 #endif
 
 
@@ -898,7 +877,6 @@ main(int argc, char *argv[])
   raptor_uri* base_uri;
   raptor_qname* el_name;
   unsigned long count;
-  raptor_uri* datatype;
   
   /* for raptor_new_iostream_to_string */
   void *string = NULL;
@@ -958,11 +936,9 @@ main(int argc, char *argv[])
 
   raptor_turtle_writer_raw_counted(turtle_writer, (const unsigned char*)" ", 1);
 
-  datatype = raptor_new_uri(world, (const unsigned char*)"http://www.w3.org/2001/XMLSchema#decimal");
   raptor_turtle_writer_literal(turtle_writer, nstack,
                                (const unsigned char*)"10.0", NULL,
-                               datatype);
-  raptor_free_uri(datatype);
+                               world->xsd_decimal_uri);
 
   raptor_turtle_writer_newline(turtle_writer);
 
@@ -983,7 +959,7 @@ main(int argc, char *argv[])
   
   count = raptor_iostream_tell(iostr);
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Freeing iostream\n", program);
 #endif
   raptor_free_iostream(iostr);
@@ -1007,7 +983,7 @@ main(int argc, char *argv[])
     return 1;
   }
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Made Turtle string of %d bytes\n", program, (int)string_len);
   fputs("[[", stderr);
   (void)fwrite(string, 1, string_len, stderr);
